@@ -320,20 +320,27 @@ class Bookmarks {
 		return true;
 	}
 
+	public function hashFolder($userId, $folderId, $fields = ['title', 'url']) {
+		$folderRecord = $this->getFolder($userId, $folderId);
+		$children = $this->getFolderChildren($userId, $folderId);
+		$childHashes = array_map(function ($item) use ($userId, $fields) {
+			switch ($item['type']) {
+				case 'bookmark':
+				  return $this->hashBookmark($userId, $item['id'], $fields);
+				case 'folder':
+				  return $this->hashFolder($userId, $item['id'], $fields);
+			  default:
+				  throw new UnexpectedValueException('Expected bookmark or folder, but not '.$item['type']);
+			}
+		}, $children);
+		$folder = ['title' => $folderRecord['title'], 'children' => $childHashes];
+		return Murmur2Hash::hash(json_encode($folder));
+	}
+
 	public function deleteFolder($userId, $folderId) {
 		$dbType = $this->config->getSystemValue('dbtype', 'sqlite');
-		$qb = $this->db->getQueryBuilder();
-		$qb
-			->delete('bookmarks_folders')
-			->where($qb->expr()->eq('id', $qb->createNamedParameter($folderId)))
-			->andWhere($qb->expr()->eq('user_id', $qb->createNamedParameter($userId)));
-		$result = $qb->execute();
 
-		if ($result === 0) {
-			return;
-		}
-
-		// get all bookmarks that are in this folder *only*
+		// get all bookmarks that are in this folder
 		$qb = $this->db->getQueryBuilder();
 		$qb
 				->select('id')
@@ -343,21 +350,7 @@ class Bookmarks {
 		$bookmarksToDelete = $qb->execute()->fetchAll(\PDO::FETCH_COLUMN);
 		// ... and "delete" them
 		foreach ($bookmarksToDelete as $bookmarkId) {
-			// remove this folder from the list of parent folders
-			$bookmark = $this->findUniqueBookmark($bookmarkId, $userId);
-			$newFolders = [];
-			foreach ($bookmark['folders'] as $oldFolderId) {
-				if ((string) $oldFolderId === (string) $folderId) {
-					continue;
-				}
-				$newFolders[] = $oldFolderId;
-			}
-			// only if no parent folders are left do we delete the bookmark as a whole
-			if (count($newFolders) > 0) {
-				$this->editBookmark($userId, $bookmarkId, $bookmark['url'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], $bookmark['public'], $newFolders);
-			} else {
-				$this->deleteUrl($userId, $bookmarkId);
-			}
+			$this->removeFromFolders($userId, $bookmarkId, [$folderId]);
 		}
 
 		// delete all subfolders
@@ -365,6 +358,14 @@ class Bookmarks {
 		foreach ($childFolders as $folder) {
 			$this->deleteFolder($userId, $folder['id']);
 		}
+
+		// delete this folder
+		$qb = $this->db->getQueryBuilder();
+		$qb
+			->delete('bookmarks_folders')
+			->where($qb->expr()->eq('id', $qb->createNamedParameter($folderId)))
+			->andWhere($qb->expr()->eq('user_id', $qb->createNamedParameter($userId)));
+		$result = $qb->execute();
 	}
 
 	private function getBookmarkParentFolders($bookmarkId) {
@@ -405,6 +406,18 @@ class Bookmarks {
 		}
 	}
 
+	public function hashBookmark($userId, $bookmarkId, $fields) {
+		$bookmarkRecord = $this->findUniqueBookmark($bookmarkId, $userId);
+		$bookmark = [];
+		foreach ($fields as $field) {
+			if (isset($bookmarkRecord[$field])) {
+				$bookmark[$field] = $bookmarkRecord[$field];
+			}
+		}
+		return Murmur2Hash::hash(json_encode($bookmark));
+	}
+
+
 	/**
 	 * @brief Check if an URL is bookmarked
 	 * @param string $url Url of a possible bookmark
@@ -665,8 +678,12 @@ class Bookmarks {
 	 * @param string $userrId User ID
 	 */
 	public function deleteAllBookmarks($userId) {
-		$allBookmarks = $this->findBookmarks($userId, -1, 'id', [], false, -1);
-		foreach ($allBookmarks as $bookmark) {
+		$rootFolders = $this->listChildFolders($userId, -1);
+		foreach ($rootFolders as $folder) {
+			$this->deleteFolder($userId, $folder['id']);
+		}
+		$restBookmarks = $this->findBookmarks($userId, -1, 'id', [], false, -1);
+		foreach ($restBookmarks as $bookmark) {
 			$this->deleteUrl($userId, $bookmark['id']);
 		}
 	}
@@ -876,7 +893,7 @@ class Bookmarks {
 		// normalize url
 		$url = $this->urlNormalizer->normalize($url);
 
-		$urlWithoutPrefix = trim(substr($url, strpos($url, "://") + 3)); // Removes everything from the url before the "://" pattern (included)
+		$urlWithoutPrefix = trim(substr($url, strpos($url, "://"))); // Removes everything from the url before the "://" pattern (excluded)
 		$decodedUrlNoPrefix = htmlspecialchars_decode($urlWithoutPrefix);
 		$decodedUrl = htmlspecialchars_decode($url);
 
